/**
 * \file    qwwad_reciprocal_fcc.cpp
 * \brief   Reciprocal Lattice Vectors-Face Centred Cubic
 * \author  Paul Harrison  <p.harrison@shu.ac.uk>
 * \author  Alex Valavanis <a.valavanis@leeds.ac.uk>
 * \details This program calculates the reciprocal lattice vectors for a
 *          face-centred cubic crystal.  The maximum magnitude of the vectors
 *          begin read in as the only command line argument.
 *
 *          The reciprocal lattice vector is assumed to be constructed from linear
 *          combinations of the reciprocal lattice basis vectors, i.e.
 *
  G=2*pi/A0*(beta_1*b_1+beta_2*b_2+beta_3*b_3)
                    ---        ---        ---

then

  G=2*pi/A0*(gamma_1*i+gamma_2*j+gamma_3*k)
                      -         -         -

where i, j and k are the cartesian basis vectors.
*/
 
#include <cstdio>
#include <cstdlib>
#include <cmath>
#include "struct.h"
#include "maths.h"

int main(int argc, char *argv[])
{
int beta_1;          /* coefficients of the reciprocal lattice vectors */
int beta_2;          /*  b_1, b_2 and b_3 constituting the general     */
int beta_3;          /*  reciprocal lattice vector G                   */
float gamma_1;
float gamma_2;
float gamma_3;
double G_max;           /* maximum value of |G|, e.g. [400]=>|G|=4        */
FILE *FG;

/* default values	*/

G_max=4;

while((argc>1)&&(argv[1][0]=='-'))
{
 switch(argv[1][1])
 {
  case 'g':
	   G_max=atof(argv[2]);
           break;
  default :
           printf("Usage:  rlv-fcc [-g maximum value of |G|]\n");
           break;
 }
 argv++;
 argv++;
 argc--;
 argc--;
}





FG=fopen("G.r","w");

for(beta_1=-G_max;beta_1<=G_max;beta_1++)
 for(beta_2=-G_max;beta_2<=G_max;beta_2++)
  for(beta_3=-G_max;beta_3<=G_max;beta_3++)
  {
   gamma_1=(float)(beta_1-beta_2+beta_3);
   gamma_2=(float)(beta_1+beta_2-beta_3);
   gamma_3=(float)(-beta_1+beta_2+beta_3);
   if(round(gamma_1*gamma_1+gamma_2*gamma_2+gamma_3*gamma_3)<=G_max*G_max)
    fprintf(FG,"%f %f %f\n",gamma_1,gamma_2,gamma_3);
  }
fclose(FG);

return EXIT_SUCCESS;
}
